#include "beer.h"
#include <stdio.h>
#include <qapplication.h>
#include <qpainter.h>

#define CLIP_POINT 120
#define FRAG_SIZE 128
#define LOG_2_FRAG_SIZE 7
#define SAMPLE_RATE 8000
#define TIMER_INTERVAL ( 1000 * FRAG_SIZE / SAMPLE_RATE )
#define WIDF 1024
#define HITE 768
#define YSCALE (HITE/128)

#define ABS( x ) ( ( ( x ) < 0 ? - ( x ) : ( x ) ) )

beer_t::beer_t( QWidget *parent = 0, const char *name = 0, WFlags f = 0 )
{
    dsp.open_device_for_read();
    dsp.set_sample_format( s8 );
    dsp.set_sample_rate( SAMPLE_RATE );
    dsp.set_num_channels( 2 );
    dsp.set_frag_size( LOG_2_FRAG_SIZE );
    timer = new QTimer();
    QObject::connect( timer, SIGNAL( timeout() ), this, SLOT( draw() ) );
    timer->start( TIMER_INTERVAL );
    resize( WIDF, HITE );
    setMinimumSize( WIDF, HITE );
    setMaximumSize( WIDF, HITE );
    clipped = false;
}

void beer_t::draw()
{
    signed char *buf, frame;
    long double sum;
    int i, frag_count;

    sum = 0;
    frag_count = 0;
    do {
        frag_count++;
        buf = ( signed char * ) dsp.get_record_buffer();
        for( i = 0; i < dsp.get_frag_size_in_bytes() / 2; i++ ) {
            sum += ( long double ) ABS( buf[i] );
            if( ABS( buf[i] ) > CLIP_POINT ) {
                clipped = true;
            }
        }
    } while( dsp.is_ready_to_give_record_buffer() );
    frame = ( signed char ) ( sum * 2 / ( dsp.get_frag_size_in_bytes() * frag_count ) );
    frame = 128 - frame;
    QPainter paint( qApp->desktop() );
    paint.setPen( black );
    if( clipped == false ) {
        paint.setBrush( QBrush( black ) );
    } else {
        paint.setBrush( QBrush( QColor( 0x70, 0, 0 ) ) );
    }
    paint.drawRect( 0, 0, WIDF, ( 128 - frame ) * YSCALE );
    paint.setPen( green );
    paint.setBrush( QColor( 0, 0x90, 0 ) );
    paint.drawRect( 0, ( 128 - frame ) * YSCALE, WIDF, frame * YSCALE );
    paint.end();
}

void beer_t::mousePressEvent( QMouseEvent *mev )
{
    clipped = false;
}

int main( int argc, char **argv )
{
    QApplication app( argc, argv );
    beer_t beer;
    app.setMainWidget( &beer );
    beer.show();
    return( app.exec() );
}

