#include "dsp_linux.h"
#include <linux/soundcard.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <stdlib.h>
#include <assert.h>

dsp_t::dsp_t()
{
    fd = -1;
    frag_size_in_bytes = 0;
    frag_buffer = NULL;
}

dsp_t::~dsp_t()
{
}

void dsp_t::open_device_for_read()
{
    fd = open( DSP_DEVICE, O_RDONLY );
    assert( fd != -1 );
}

void dsp_t::open_device_for_write()
{
    fd = open( DSP_DEVICE, O_WRONLY );
    assert( fd != -1 );
}

void dsp_t::close_device()
{
    close( fd );
}

void dsp_t::set_sample_format( enum sample_format_t sample_format )
{
    int fmt;

    fmt = ( int ) sample_format;
    assert( ioctl( fd, SNDCTL_DSP_SETFMT, &fmt ) != -1 );
}

void dsp_t::set_sample_rate( int sample_rate )
{
    assert( ioctl( fd, SNDCTL_DSP_SPEED, &sample_rate ) != -1 );
}

void dsp_t::set_num_channels( int num_channels )
{
    assert( ioctl( fd, SNDCTL_DSP_CHANNELS, &num_channels ) != -1 );
}

void dsp_t::set_frag_size( unsigned short two_to_the )
{
    int blah;

    blah  = (2 << 16) + two_to_the;  // 2 buffers, each 2^two_to_the bytes big
    assert( ioctl( fd, SNDCTL_DSP_SETFRAGMENT, &blah ) != -1 );
    frag_size_in_bytes = 1 << two_to_the;
    if( frag_buffer != NULL ) {
        free( frag_buffer );
    }
    frag_buffer = ( unsigned char * ) malloc( frag_size_in_bytes );
}

bool dsp_t::is_ready_for_play_buffer()
{
    audio_buf_info info;

    assert( ioctl( fd, SNDCTL_DSP_GETOSPACE, &info ) != -1 );
    return( info.fragments > 0 );
}

void* dsp_t::get_play_buffer()
{
    return( frag_buffer );
}

void dsp_t::send_play_buffer()
{
    assert( write( fd, frag_buffer, frag_size_in_bytes ) != -1 );
}

bool dsp_t::is_ready_to_give_record_buffer()
{
    audio_buf_info info;

    assert( ioctl( fd, SNDCTL_DSP_GETISPACE, &info ) != -1 );
    return( info.fragments > 0 );
}

void* dsp_t::get_record_buffer()
{
    assert( read( fd, frag_buffer, frag_size_in_bytes ) != -1 );
    return( frag_buffer );
}

void dsp_t::read_buf( void *outbuf, int num_bytes )
{
    assert( read( fd, outbuf, num_bytes ) != -1 );
}
